@@ -36,6 +36,8 @@ from pydal.dbapi import RestAPI, Policy
 policy = Policy()
 policy.set('superhero', 'GET', authorize=True, allowed_patterns=['*'])
 policy.set('*', 'GET', authorize=True, allowed_patterns=['*'])
+
+# for security reasons we disabled here all methods but GET at the policy level, to enable any of them just set authorize = True
 policy.set('*', 'PUT', authorize=False)
 policy.set('*', 'POST', authorize=False)
 policy.set('*', 'DELETE', authorize=False)
@@ -58,8 +60,8 @@ The above action is exposed as:
 ``
 /superheroes/rest/api/{tablename}
 ``    
-
-In our example policy we disabled all methods but GET.
+    
+**About request.POST**: keep in mind that **request.POST** only contains the form data that is posted using a **regular HTML-form** or javascript **FormData** object. If you post just plain object (e.g. ``axios.post( 'path/to/api', {field:'some'} )``) you should pass **request.json** instead of request.POST,  since latter will contain just raw request-body which is string, not json. See bottle.py documentation for more details.    
 
 #### RestAPI GET
 
@@ -123,7 +125,9 @@ Lookup denormalizes the linked field.
 Here are some practical examples:
 
 URL:
-``/superheroes/rest/api/superhero``
+``
+/superheroes/rest/api/superhero
+``
 
 OUTPUT:
 ``
@@ -587,9 +591,10 @@ OUTPUT:
 }
 ``
 
-URL:
+URL (it's a single line):
 ``
-/superheroes/rest/api/superhero?@lookup=powers:superhero.tag[strength].superpower[description]
+/superheroes/rest/api/superhero?
+@lookup=powers:superhero.tag[strength].superpower[description]
 ``
 
 OUTPUT:
@@ -686,9 +691,10 @@ OUTPUT:
 }
 ``
 
-URL:
+URL (it's a single line):
 ``
-/superheroes/rest/api/superhero?@lookup=powers!:superhero.tag[strength].superpower[description]
+/superheroes/rest/api/superhero?
+@lookup=powers!:superhero.tag[strength].superpower[description]
 ``
 
 OUTPUT:
@@ -765,9 +771,10 @@ OUTPUT:
 }
 ``
 
-URL:
+URL (it's a single line):
 ``
-/superheroes/rest/api/superhero?@lookup=powers!:superhero.tag[strength].superpower[description],identity!:real_identity[name]
+/superheroes/rest/api/superhero?
+@lookup=powers!:superhero.tag[strength].superpower[description],identity!:real_identity[name]
 ``
 
 OUTPUT:
