@@ -19,10 +19,10 @@ DB_MIGRATE = True
 DB_FAKE_MIGRATE = False  # maybe?
 
 # location where to store uploaded files:
-UPLOAD_PATH = os.path.join(APP_FOLDER, "uploads")
+UPLOAD_FOLDER = os.path.join(APP_FOLDER, "uploads")
 
 # send email on regstration
-VERIFY_EMAIL = False
+VERIFY_EMAIL = True
 
 # account requires to be approved ?
 REQUIRES_APPROVAL = False
@@ -36,7 +36,7 @@ SMTP_TLS = False
 
 # session settings
 SESSION_TYPE = "cookies"
-SESSION_SECRET_KEY = "<my secret key>"
+SESSION_SECRET_KEY = "<session-secret-key>" # replace this with a uuid
 MEMCACHE_CLIENTS = ["127.0.0.1:11211"]
 REDIS_SERVER = "localhost:6379"
 
@@ -80,5 +80,5 @@ CELERY_BROKER = "redis://localhost:6379/0"
 # try import private settings
 try:
     from .settings_private import *
-except:
+except (ImportError, ModuleNotFoundError):
     pass
